@@ -1,10 +1,12 @@
 from __future__ import print_function, division, absolute_import
 
+from collections import deque
 import logging
+from ..metrics import time
 
 from tornado import gen
 
-from ..utils import log_errors, PeriodicCallback
+from ..utils import log_errors, PeriodicCallback, parse_timedelta
 
 logger = logging.getLogger(__name__)
 
@@ -20,11 +22,20 @@ class Adaptive(object):
     scheduler: distributed.Scheduler
     cluster: object
         Must have scale_up and scale_down methods/coroutines
-    startup_cost : int, default 1
-        Factor representing how costly it is to start an additional worker.
+    startup_cost : timedelta or str, default "1s"
+        Estimate of the number of seconds for nnFactor representing how costly it is to start an additional worker.
         Affects quickly to adapt to high tasks per worker loads
+    interval : timedelta or str, default "1000 ms"
+        Milliseconds between checks
+    wait_count: int, default 3
+        Number of consecutive times that a worker should be suggested for
+        removal before we remove it.
     scale_factor : int, default 2
         Factor to scale by when it's determined additional workers are needed
+    minimum: int
+        Minimum number of workers to keep around
+    maximum: int
+        Maximum number of workers to keep around
     **kwargs:
         Extra parameters to pass to Scheduler.workers_to_close
 
@@ -39,7 +50,7 @@ class Adaptive(object):
     Notes
     -----
     Subclasses can override :meth:`Adaptive.should_scale_up` and
-    :meth:`Adaptive.should_scale_down` to control when the cluster should be
+    :meth:`Adaptive.workers_to_close` to control when the cluster should be
     resized. The default implementation checks if there are too many tasks
     per worker or too little memory available (see :meth:`Adaptive.needs_cpu`
     and :meth:`Adaptive.needs_memory`).
@@ -48,16 +59,23 @@ class Adaptive(object):
     the cluster's ``scale_up`` method.
     '''
 
-    def __init__(self, scheduler, cluster, interval=1000, startup_cost=1,
-                 scale_factor=2, **kwargs):
+    def __init__(self, scheduler, cluster, interval='1s', startup_cost='1s',
+                 scale_factor=2, minimum=0, maximum=None, wait_count=3,
+                 **kwargs):
+        interval = parse_timedelta(interval, default='ms')
         self.scheduler = scheduler
         self.cluster = cluster
-        self.startup_cost = startup_cost
+        self.startup_cost = parse_timedelta(startup_cost, default='s')
         self.scale_factor = scale_factor
-        self._adapt_callback = PeriodicCallback(self._adapt, interval)
+        self._adapt_callback = PeriodicCallback(self._adapt, interval * 1000)
         self.scheduler.loop.add_callback(self._adapt_callback.start)
         self._adapting = False
         self._workers_to_close_kwargs = kwargs
+        self.minimum = minimum
+        self.maximum = maximum
+        self.log = deque(maxlen=1000)
+        self.close_counts = {}
+        self.wait_count = wait_count
 
     def needs_cpu(self):
         """
@@ -114,6 +132,7 @@ class Adaptive(object):
         1. There are unrunnable tasks and no workers
         2. The cluster is CPU constrained
         3. The cluster is RAM constrained
+        4. There are fewer workers than our minimum
 
         See Also
         --------
@@ -121,6 +140,12 @@ class Adaptive(object):
         needs_memory
         """
         with log_errors():
+            if len(self.scheduler.workers) < self.minimum:
+                return True
+
+            if self.maximum is not None and len(self.scheduler.workers) >= self.maximum:
+                return False
+
             if self.scheduler.unrunnable and not self.scheduler.workers:
                 return True
 
@@ -132,27 +157,6 @@ class Adaptive(object):
 
             return False
 
-    def should_scale_down(self):
-        """
-        Determine whether any workers should potentially be removed from
-        the cluster.
-
-        Returns
-        -------
-        scale_down : bool
-
-        Notes
-        -----
-        ``Adaptive.should_scale_down`` defaults to dispatching to
-        ``Adaptive.workers_to_close``, returning True if any workers to close
-        are specified.
-
-        See Also
-        --------
-        Scheduler.workers_to_close
-        """
-        return len(self.workers_to_close()) > 0
-
     def workers_to_close(self, **kwargs):
         """
         Determine which, if any, workers should potentially be removed from
@@ -167,27 +171,40 @@ class Adaptive(object):
         ``Adaptive.workers_to_close`` dispatches to Scheduler.workers_to_close(),
         but may be overridden in subclasses.
 
+        Returns
+        -------
+        List of worker addresses to close, if any
+
         See Also
         --------
         Scheduler.workers_to_close
         """
+        if len(self.scheduler.workers) <= self.minimum:
+            return []
         kw = dict(self._workers_to_close_kwargs)
         kw.update(kwargs)
-        return self.scheduler.workers_to_close(**kw)
+        L = self.scheduler.workers_to_close(**kw)
+        if len(self.scheduler.workers) - len(L) < self.minimum:
+            L = L[:len(self.scheduler.workers) - self.minimum]
+        return L
 
     @gen.coroutine
-    def _retire_workers(self):
+    def _retire_workers(self, workers=None):
+        if workers is None:
+            workers = self.workers_to_close()
         with log_errors():
-            workers = yield self.scheduler.retire_workers(workers=self.workers_to_close(),
-                                                          remove=True,
-                                                          close_workers=True)
+            result = yield self.scheduler.retire_workers(workers=workers,
+                                                         remove=True,
+                                                         close_workers=True)
 
-            if workers:
-                logger.info("Retiring workers %s", workers)
-                f = self.cluster.scale_down(workers)
+            if result:
+                logger.info("Retiring workers %s", result)
+                f = self.cluster.scale_down(result)
                 if gen.is_future(f):
                     yield f
 
+            raise gen.Return(result)
+
     def get_scale_up_kwargs(self):
         """
         Get the arguments to be passed to ``self.cluster.scale_up``.
@@ -205,6 +222,8 @@ class Adaptive(object):
         """
         instances = max(1, len(self.scheduler.workers) * self.scale_factor)
         logger.info("Scaling up to %d workers", instances)
+        if self.maximum:
+            instances = min(self.maximum, instances)
         return {'n': instances}
 
     @gen.coroutine
@@ -215,18 +234,35 @@ class Adaptive(object):
         self._adapting = True
         try:
             should_scale_up = self.should_scale_up()
-            should_scale_down = self.should_scale_down()
-            if should_scale_up and should_scale_down:
+            workers = set(self.workers_to_close())
+            if should_scale_up and workers:
                 logger.info("Attempting to scale up and scale down simultaneously.")
-            else:
-                if should_scale_up:
-                    kwargs = self.get_scale_up_kwargs()
-                    f = self.cluster.scale_up(**kwargs)
-                    if gen.is_future(f):
-                        yield f
+                return
 
-                if should_scale_down:
-                    yield self._retire_workers()
+            if should_scale_up:
+                kwargs = self.get_scale_up_kwargs()
+                f = self.cluster.scale_up(**kwargs)
+                self.log.append((time(), 'up', kwargs))
+                if gen.is_future(f):
+                    yield f
+
+            d = {}
+            to_close = []
+            for w, c in self.close_counts.items():
+                if w in workers:
+                    if c >= self.wait_count:
+                        to_close.append(w)
+                    else:
+                        d[w] = c
+
+            for w in workers:
+                d[w] = d.get(w, 0) + 1
+
+            self.close_counts = d
+
+            if to_close:
+                self.log.append((time(), 'down', workers))
+                workers = yield self._retire_workers(workers=to_close)
         finally:
             self._adapting = False
 
