@@ -2,7 +2,7 @@ from __future__ import print_function, division, absolute_import
 
 import logging
 from tornado import gen
-from .client import futures_of, _wait
+from .client import futures_of, wait
 from .utils import sync, tokey
 from .utils_comm import pack_data
 from .worker import _deserialize
@@ -44,12 +44,13 @@ class ReplayExceptionScheduler(object):
             if isinstance(key, list):
                 key = tuple(key)  # ensure not a list from msgpack
             key = tokey(key)
-            if key in self.scheduler.exceptions_blame:
-                cause = self.scheduler.exceptions_blame[key]
-                # cannot serialize sets
-                return {'deps': list(self.scheduler.dependencies[cause]),
-                        'cause': cause,
-                        'task': self.scheduler.tasks[cause]}
+            ts = self.scheduler.tasks.get(key)
+            if ts is not None and ts.exception_blame is not None:
+                cause = ts.exception_blame
+                # NOTE: cannot serialize sets
+                return {'deps': [dts.key for dts in cause.dependencies],
+                        'cause': cause.key,
+                        'task': cause.run_spec}
 
 
 class ReplayExceptionClient(object):
@@ -124,7 +125,7 @@ class ReplayExceptionClient(object):
 
     @gen.coroutine
     def _recreate_error_locally(self, future):
-        yield _wait(future)
+        yield wait(future)
         out = yield self._get_futures_error(future)
         function, args, kwargs, deps = out
         futures = self.client._graph_to_futures({}, deps)
