@@ -10,7 +10,7 @@ from tornado import gen
 from distributed import Client, Queue, Nanny, worker_client, wait
 from distributed.metrics import time
 from distributed.utils_test import (gen_cluster, inc, cluster, slow, div)
-from distributed.utils_test import loop # flake8: noqa
+from distributed.utils_test import loop # noqa: F401
 
 
 @gen_cluster(client=True)
@@ -33,11 +33,11 @@ def test_queue(c, s, a, b):
     del future, future2
 
     yield gen.sleep(0.1)
-    assert s.task_state  # future still present in y's queue
+    assert s.tasks  # future still present in y's queue
     yield y.get()  # burn future
 
     start = time()
-    while s.task_state:
+    while s.tasks:
         yield gen.sleep(0.01)
         assert time() < start + 5
 
@@ -236,3 +236,6 @@ def test_erred_future(c, s, a, b):
     future2 = yield q.get()
     with pytest.raises(ZeroDivisionError):
         yield future2.result()
+
+    exc = yield future2.exception()
+    assert isinstance(exc, ZeroDivisionError)
